﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Net.Http.Formatting;
using System.Threading.Tasks;

namespace FMK.EFH.API.Sample
{
    public class ApiRequest
    {
        public static T Process<T>(Func<Task<HttpResponseMessage>> task)
        {
            byte[] dummyVar;
            return Process<T>(task, false, false, out dummyVar);
        }

        public static void Process(Func<Task<HttpResponseMessage>> task)
        {
            byte[] dummyVar;
            Process(task, false, false, out dummyVar);
        }

        private static T Process<T>(Func<Task<HttpResponseMessage>> task, bool noResponse, bool saveRequest, out byte[] requestData)
        {
            requestData = null;
            var response = task.Invoke().Result;

            if (!response.IsSuccessStatusCode)
            {
                var requestModel = response.Content.ReadAsAsync<ApiExceptionModel>(GetResponseFormatters()).Result;

                if (requestModel != null)
                    throw new ApiException(requestModel.Message, (HttpStatusCode)response.StatusCode) {Model = requestModel};
                else
                {
                    response.EnsureSuccessStatusCode(); //Throw default exception when no model can be parsed.
                }
            }

            if (noResponse)
                return default(T);

            var obj = response.Content.ReadAsAsync<T>(GetResponseFormatters()).Result;

            if (saveRequest)
                requestData = response.Content.ReadAsByteArrayAsync().Result;
            return obj;
        }

        private static void Process(Func<Task<HttpResponseMessage>> task, bool noResponse, bool saveRequest, out byte[] requestData)
        {
            requestData = null;
            var response = task.Invoke().Result;

            if (!response.IsSuccessStatusCode)
            {
                var requestModel = response.Content.ReadAsAsync<ApiExceptionModel>(GetResponseFormatters()).Result;

                if (requestModel != null)
                    throw new ApiException(requestModel.Message, (HttpStatusCode)response.StatusCode) { Model = requestModel };
                else
                {
                    response.EnsureSuccessStatusCode(); //Throw default exception when no model can be parsed.
                }
            }

            if (noResponse)
                return;
            
            if (saveRequest)
                requestData = response.Content.ReadAsByteArrayAsync().Result;
        }

        private static List<MediaTypeFormatter> GetResponseFormatters()
        {
            var formatters = new List<MediaTypeFormatter>();
            var xmlFormatter = new XmlMediaTypeFormatter();
            formatters.Add(xmlFormatter);
            return formatters;
        }
    }
}